/************************ HOUSE2.C ***********************
 *  A demonstration of drawing scaling using a World
 *  Coordinate System. Rather than real numbers, longs
 *  are used. A value of 1 is stored as 1000. 50% becomes
 *  500, 1% becomes 10, etc.
 *
 * Author: John M. Dlugosz (port to Turbo C by John Rex)
 ********************************************************/

#include <graphics.h>
#include <math.h>
#include <conio.h>
#include <stdlib.h>

typedef double real;
typedef int world;

typedef struct {
   world x_org, y_org;
   real x_scale, y_scale;
} world_rec;

world_rec my_world;

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void transform (world x_in, world y_in, world *x_out, world *y_out)
{
    *x_out = (world) (x_in * my_world.x_scale) + my_world.x_org;
    *y_out = (world) (y_in * my_world.y_scale) + my_world.y_org;
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */
void setup_my_world()
/* the origin is (0,0) and the scale is set so that the height is 1000 (100%)
   the width is such that squares are square and circles are round.  */
{
    real adjustment;
    world yasp,xasp;

    my_world.x_org = 0;
    my_world.y_org = 0;
    /* when transforming, a y_in of 1000 should give a y_out of maxy.
       maxy = 1000 * my_world.y_scale + 0;  so... */
    my_world.y_scale = (real) getmaxy() / 1000.0;

    /* now figure the x_scale so aspect ratio of world is 1 */
    getaspectratio (&xasp, &yasp);
    adjustment = (real) yasp / (real) xasp;

    /* in the previous programs, this adjustment value multiplied by a Y value
       gave a 'square' x value.  So if I transform an x_in of 1000, I want
       maxy*adjustment out.  maxy*adjustment = 1000 * x_scale + 0;   so... */

    my_world.x_scale = (real) getmaxy() * adjustment / 1000.0;
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void SetWorld (world x1, world y1, world x2, world y2, world_rec *saveworld)
/* given a box (as ll and ur points) in the current world, make the
   box the entire world (1000 by 1000 now fills the box).  The old
   world is saved by the caller in saveworld.  */
{
    world_rec newworld;
    world mappedX, mappedY;

    transform (x1,y1, &mappedX,&mappedY);
    newworld.x_org = mappedX;
    newworld.y_org = mappedY;
    transform (x2,y2, &mappedX,&mappedY);

    /* an x_in of 1000 should yield an x_out of mappedX
       mappedX = 1000 * x_scale + x_org;      */
    newworld.x_scale = (real) (mappedX-newworld.x_org) /1000.0;
    /* and likewise for Y */
    newworld.y_scale = (real) (mappedY-newworld.y_org) /1000.0;

    /* now do the switch */
    *saveworld = my_world;
    my_world = newworld;
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void RestoreWorld (world_rec *world)
{
    my_world = *world;
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void drawline (world x1, world y1, world x2, world y2)
{
    world mappedX, mappedY, mappedX2, mappedY2;

    transform (x1,y1, &mappedX,&mappedY);
    transform (x2,y2, &mappedX2,&mappedY2);
    line (mappedX,mappedY, mappedX2,mappedY2);
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void drawrec (world x1, world y1, world x2, world y2)
{
    world mappedX, mappedY, mappedX2, mappedY2;

    transform (x1,y1, &mappedX,&mappedY);
    transform (x2,y2, &mappedX2,&mappedY2);
    rectangle (mappedX,mappedY,mappedX2,mappedY2);
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void WaitForKeypressWithoutTrashingGraphicsScreen()
{
    (void) getch();
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

int startupgraphics()
{
    char path[100];
    int graphdriver,graphmode;
    void bgiparse(char *, char *, int *, int *);

    bgiparse(getenv("BGI"), path, &graphdriver, &graphmode);
    initgraph (&graphdriver, &graphmode, path);
    return (graphresult() == 0);
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */
void draw_body (world x1, world y1, world x2, world y2)
/* draw body inside the passed rectangle */
{
    world_rec oldworld;
    SetWorld (x1,y1,x2,y2, &oldworld);
    drawrec (0,0,1000,1000);  /* the big box */
    /* the size of the door is 20% by 50%.  It is bottom center */
    drawrec (400,1000, 600,500);
    /* windows are 200 by 200, located 100 from corners */
    drawrec (100,100, 300,300);
    drawrec (700,100, 900,300);
    RestoreWorld (&oldworld);
}
/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void drawhouse (world x1, world y1, world x2, world y2)
/* draws a house to exactly fit inside the rectangle */
{
    world_rec oldworld;
    SetWorld (x1,y1,x2,y2, &oldworld);
    /* roof is 1/3 the way down, at 333 */
    draw_body (0,333, 1000,1000);
    drawline (0,333, 500,0);    /* left side of roof */
    drawline (500,0,1000,333);  /* right side of roof */
    RestoreWorld(&oldworld);
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void show()
{
    drawhouse (700,10, 1033,510);
    drawhouse (50,50, 650,950);
    WaitForKeypressWithoutTrashingGraphicsScreen();
}

/* /\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\ */

void main()
{
   if (startupgraphics()) {
       setup_my_world();
       show();
       closegraph();
   }
   else printf("Error going into graphics mode.\n");
}